//	compansi
//
//	ANSI test file for converting data
//
//	Self-contained for easy publishing

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <ctype.h>

#include "PDB.h"
#include "NBuddyPDBFormat.h"
#include "StringTables.h"

char *months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", 
						"Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

void main(void);
void export(FILE *fs, FILE *fd);
void setshort(unsigned short *s, unsigned short value);
void setlong(unsigned long *l, unsigned long value);
long getlong(void *longp);
short getshort(void *shortp);
void fgetstring(FILE *fs, char *string);
time_t	PilotToTimeT(void);

void main(void)
{
	FILE	*fs;
	char	filename[80];

RESTART:
	printf("Tide filename to convert: ");
	if (fgets(filename,80,stdin) == NULL)
		return;
		
	sscanf(filename, "%s\n", filename);
	
	fs = fopen(filename, "r+b");
	if (fs == NULL)
	{
		printf("ERROR: Couldn't open %s\n", filename);
		goto RESTART;
	}
	
	export(fs, stdout);
	
	fclose(fs);
	
	printf("File converted.\n\n");
	
	printf("Press Return/Enter to exit.\n");
	fgets(filename,80,stdin);
}

void export(FILE *fs, FILE *fd)
{
	short					numRecords,i;
	time_t					pilotTimeDiff;
	PDBHeader				pdbh;
	RollHeaderPackedRec		rollheader;

	pilotTimeDiff = PilotToTimeT();

	fseek(fs, 0, SEEK_SET);
	fread(&pdbh, sizeof(pdbh), 1, fs);

	if (pdbh.version != 0x0000)
	{
		printf("ERROR: Unknown PDB version\n");
		return;
	}
		
	numRecords = getshort(&(pdbh.numRecords));
	
	for (i = 0; i < numRecords; i++)
	{
		PDBRecordEntry	record;
		long			offset;
		time_t			datetime;
		short			rollNumber;
		short			frames;
		short			frame;
		short			storageLevel;
		struct tm		*curtime;
		char			camera[256];
		char			caption[256];
		

		fseek(fs, kPDBHeaderSize + kPDBRecordEntrySize * i, SEEK_SET);
		fread(&record, kPDBRecordEntrySize, 1, fs);

		if (record.attr == 0)
		{
			offset = getlong(&(record.offset));
		
			fseek(fs, offset, SEEK_SET);
			
			fread(&rollheader, kRollHeaderPackedSize, 1, fs);
			fgetstring(fs, camera);
			fgetstring(fs, caption);
			
			rollNumber = getshort(&(rollheader.rollNumber));
			datetime = getlong(&(rollheader.timeDate)) + pilotTimeDiff;
			curtime = localtime(&datetime);
			storageLevel = rollheader.storageLevel;
			frames = rollheader.frameCount;
			
			fprintf(fd, "Roll: %04d - %s\n", rollNumber, caption);
			fprintf(fd, "Date: %d %s %d, %d:%02d\n", curtime->tm_mday, months[curtime->tm_mon], curtime->tm_year + 1900,
									curtime->tm_hour, curtime->tm_min);
			fprintf(fd, "ISO: %s\n", GetStringTable(kISOTable, rollheader.iso));
			fprintf(fd, "Camera: %s\n", camera);
			fprintf(fd, "Storage Level: %s\n", (storageLevel == 0) ? "Minimum" :
						(storageLevel == 1) ? "Medium" : "Maximum");

						
			for (frame = 1; frame <= frames; frame++)
			{
				FrameHeaderPackedRec	frameheader;
				
				fread(&frameheader, kFrameHeaderPackedSize, 1, fs);
				
				//		  Frame: XX  SPEED APERTURE
				fprintf(fd, "  Frame: %02d  %s\", f/%s", frame, 
					GetStringTable(kShutterSpeedTable, FHGetShutterIndex(&frameheader)),
					GetStringTable(kApertureTable, FHGetApertureIndex(&frameheader)));
					
				// +/- EV
				if ((storageLevel == 2) && (FHGetExpCompIndex(&frameheader)))
					fprintf(fd, " %s", GetStringTable(kExpCompTable, FHGetExpCompIndex(&frameheader)));
					
				// LENS EXPOSURE_MODE METER_MODE
				if (storageLevel > 0)
					fprintf(fd, ", %smm, %s, %s",
						GetStringTable(kFocalTable, FHGetFocalIndex(&frameheader)),
						GetStringTable(kExposureModeTable, FHGetExposureIndex(&frameheader)),
						GetStringTable(kMeterTable, FHGetMeterIndex(&frameheader)));
						
				// FLASH_MODE FLASH_EV
				if ((storageLevel > 0) && (FHGetFlashIndex(&frameheader) != kNoFlashIndex))
				{
					fprintf(fd, ", %s", GetStringTable(kFlashSyncTable, FHGetFlashIndex(&frameheader)));
					if ((storageLevel == 2) && (FHGetFlashCompIndex(&frameheader)))
						fprintf(fd, " %s", GetStringTable(kExpCompTable, FHGetFlashCompIndex(&frameheader)));
				}
				
				fgetstring(fs, caption);
				
				if (*caption)
					fprintf(fd, " - %s", caption);
					
				fprintf(fd, "\n");
				


			}
			fprintf(fd, "-------\n");
		
		}
	}
}

short getshort(void *shortp)
{
	unsigned char *p;
	short	result;
	
	p = shortp;
	result = ((short) *p++) << 8;
	result += *p++;
	
	return result;
}

long getlong(void *longp)
{
	unsigned char *p;
	long	result;
	
	p = longp;
	result = ((long) *p++) << 24;
	result += ((long) *p++) << 16;
	result += ((long) *p++) << 8;
	result += *p++;
	
	return result;
}


void setlong(unsigned long *l, unsigned long value)
{
	unsigned char	*p;
	
	p = (unsigned char *) l;
	*p++ = value>>24;
	*p++ = value>>16;
	*p++ = value>>8;
	*p++ = value;
}

void setshort(unsigned short *s, unsigned short value)
{
	unsigned char	*p;
	
	p = (unsigned char *) s;
	*p++ = value>>8;
	*p++ = value;
}

void fgetstring(FILE *fs, char *string)
{
	int		c;
	char	*p;
	
	p = string;
	while( ((c = fgetc(fs)) != EOF) && (c != 0))
	{
		if ((c == 0x13) || (c == 0x0A))
			c = ' ';
		*p++ = c;
	}
	*p = 0;
	
}

time_t	PilotToTimeT(void)
{
	struct tm			pilot, lib;
	
	memset(&pilot, 0, sizeof(struct tm));
	memset(&lib, 0, sizeof(struct tm));
	
	//	Pilot time is based on 1/1/1904
	pilot.tm_year = 4;
	pilot.tm_mon = 1;
	pilot.tm_mday = 1;
	
	
#if (__dest_os == __win32_os)

	//	Windows time_t is 1/1/1970
	
	lib.tm_year = 70;
	lib.tm_mon = 1;
	lib.tm_mday = 1;
#elif (__dest_os == __mac_os)

	//	Macintosh time_t is 1/1/1900
	lib.tm_year = 0;
	lib.tm_mon = 1;
	lib.tm_mday = 1;
#else

	//	Who knows?  Assume 1/1/1900
	lib.tm_year = 0;
	lib.tm_mon = 1;
	lib.tm_mday = 1;
#endif

	return(difftime(mktime(&pilot), mktime(&lib)));

}
