
#define kMaxPDBNameSize		32
#define kPDBNameSuffix		".PDB"
#define kDOSFilenameSize	12

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=packed
#endif

typedef struct PDBHeader
{
	char			name[kMaxPDBNameSize];
	unsigned short	flags;
	unsigned short	version;
	unsigned long	creationTime;
	unsigned long	modificationTime;
	unsigned long	backupTime;
	unsigned long	modificationNumber;
	unsigned long	appInfoOffset;
	unsigned long	sortInfoOffset;
	unsigned long	type;
	unsigned long	creator;
	unsigned long	uniqueID;
	unsigned long	nextRecordID;
	unsigned short	numRecords;
} PDBHeader;

#define kPDBHeaderSize	78

typedef struct PDBResourceEntry
{
	unsigned long	type;
	unsigned short	id;
	unsigned long	offset;
} PDBResourceEntry;

#define kPDBResourceEntrySize	10

typedef struct PDBRecordEntry
{
	unsigned long	offset;
	unsigned char	attr;
	unsigned long	uniqueID:24;
} PDBRecordEntry;

#define kPDBRecordEntrySize	8

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

/*	PDB Format

	PBDHeader
	PDBResourceEntry|PDBRecordEntry [numRecords]
	2 bytes 
	AppInfo (if applicable)
	SortInfo (if applicable)
	DATA ENTRIES
*/

#define kPDBFirstResourceOffset	(kPDBHeaderSize + kPDBResourceEntrySize + 2)
#define kPDBFirstRecordOffset	(kPDBHeaderSize + kPDBRecordEntrySize + 2)

enum PDBFlags {
	pdbResourceFlag = 0x0001,
	pdbReadOnlyFlag = 0x0002,
	pdbAppInfoDirtyFlag = 0x0004,
	pdbBackupFlag = 0x0008,
	pdbOpenFlag = 0x8000
};

enum PDBVersion {
	pdbVerReadOnly = 0x0001,
	pdbVerWrite = 0x0002,
	pdbVerReadWrite = 0x0003,
	pdbVerLeaveOpen = 0x0004,
	pdbVerExclusive = 0x0008,
	pdbVerShowSecret = 0x0010
};